// Sample script based action types for Trespasser CE
//
// Disclaimer: They haven't been tested and may contain bugs.

// script version of ATX ActionType 113 - Set player AI type
class ATX113 extends ActionType
{
	nvram = null;

	iAIType = EAIRefType.eaiHUMAN;

	function OnReset()
	{
		nvram.iPlayerAIType = -1;
		SetPlayerAIType(EAIRefType.eaiHUMAN);
	}

	function OnScriptInit(triggerOwner)
	{
		// Because the AI type is a setting that the game doesn't save in savegames, we need to do a bit of a hackish solution here
		// and make the AI save and restore the player's current AI type. If the there are multiple instances of this AT they will
		// all save and restore the current type, but that's ok since they should all be saving and loading the same value. It would
		// also be possible to handle it more elegantly by making a custom player script which saves and restores the AI type or a
		// simple level script with sole purpose of saving/restoring the player's AI type, but this version works as a standalone solution.
		nvram = {};
		nvram.iPlayerAIType <- -1;

		// get the AI type that this action should set from TScript property
		iAIType = GetPropInt(esFrame, EAIRefType.eaiHUMAN);
	}

	function Run()
	{
		SetPlayerAIType(iAIType);
	}

	function OnPrepareSave()
	{
		// update nvram.iPlayerAIType to the player's current AI type before saving
		nvram.iPlayerAIType = GetPlayerAIType();
	}

	function OnPostLoad(bSaveGame)
	{
		// update the player's current AI type with the state loaded from a savegame
		if (nvram.iPlayerAIType != -1)
			SetPlayerAIType(type);
	}

	// LOCAL METHODS -----------------------------------------------------------------------------

	function SetPlayerAIType(type)
	{
		// The AI type is not set on the main object, that goes for both dinos and the player, it's set on the CBoundaryBox objects.
		// There are up to 8 bboxes, the first EBoundaryBoxes.ebbBODY and last EBoundaryBoxes.ebbFLEFT_FOOT.
		local i;
		local player = TP_Engine.GetPlayer();
		for (i=EBoundaryBoxes.ebbBODY; i<=EBoundaryBoxes.ebbFLEFT_FOOT; i++)
		{
			local pbb = player.pbbBoundaryBox(i);
			if (pbb)
				pbb.AISetRefType(iAIType);
		}
	}

	function GetPlayerAIType()
	{
		// get the type from the first bbox that has one
		local i;
		local player = TP_Engine.GetPlayer();
		for (i=EBoundaryBoxes.ebbBODY; i<=EBoundaryBoxes.ebbFLEFT_FOOT; i++)
		{
			local pbb = player.pbbBoundaryBox(i);
			if (pbb)
			{
				local type = pbb.AIGetRefType();
				if (type != null)
					return type;
			}
		}

		// for unknown reason it failed to get the type, default to human
		return EAIRefType.eaiHUMAN;
	}
}


// change a Dino's AI type to human and put it on the same team as the player
// (To truly be on the same iTeam as the player, both player and dino have to on an iTeam other than 0,
// which is the default for the player. 0 has special meaning as a free-for-all team in the AI code.)
class DomesticateAT extends ActionType
{
	nvram = null;

	u4Dino = null;

	iDefaultTeam = 0;
	eaiDefaultRefType = -1;

	function OnReset()
	{
		nvram.iAIType = -1;

		if (eaiDefaultRefType != -1)
			SetAITypeAndTeam(paniDino, iDefaultTeam, eaiDefaultRefType)
	}

	function OnScriptInit(triggerOwner)
	{
		// see notes in ATX113 OnScriptInit above, the same applies to dinos and the team value. The hack to save the team and type
		// could be easily avoided by making a custom CAnimal script that saves those instead. It would make this AT much cleaner.
		nvram = {};
		nvram.iAIType <- -1;
		nvram.iTeam <- 15;

		// get target dino
		local strDino = GetPropString(esDinosaur);
		if (strDino)
			u4Dino = TP_Engine.uid(strDino);
		else
			print("DomesticateAT init failed, missing 'Dinosaur' property");
	}

	function Run()
	{
		Domesticate( paniGetDino() );
	}

	function OnPrepareSave()
	{
		local paniDino = paniGetDino();
		if (paniDino)
		{
			// update nvram to the dino's current settings
			nvram.iAIType = GetAIType(paniDino);
			nvram.iTeam = paniDino.iTeam;
		}
	}

	function OnPostLoad(bSaveGame)
	{
		// if a fresh level was loaded for the first time then remember the dino's default AI type and team
		// (otherwise the dino's type and team settings don't reset after level restarts and savegame reloads)
		if (!bSaveGame && eaiDefaultRefType != -1)
		{
			local paniDino = paniGetDino();
			if (paniDino)
			{
				iDefaultTeam = paniDino.iTeam;
				eaiDefaultRefType = GetAIType(paniDino);
			}
		}

		// update the dino's current AI type and team with the state loaded from a savegame (if anything was loaded)
		if (nvram.iAIType != -1)
			SetAITypeAndTeam(paniGetDino(), nvram.iAIType, nvram.iTeam);
	}

	// LOCAL METHODS -----------------------------------------------------------------------------

	function Domesticate(paniDino)
	{
		if (paniDino)
			SetAITypeAndTeam(paniDino, TP_Engine.GetPlayer().iTeam, EAIRefType.eaiHUMAN);
	}

	// returns a CAnimation or null
	function paniGetDino()
	{
		if (!u4Dino)
			return null;

		local pinsDino = TP_Engine.FindInstance(u4Dino);
		if (pinsDino instanceof CAnimal)
			return pinsDino;

		return null;
	}

	function SetAITypeAndTeam(paniDino, iTeam, eaiRefType)
	{
		if (!paniDino)
			return;

		paniDino.iTeam = iTeam;

		local i;
		for (i=EBoundaryBoxes.ebbBODY; i<=EBoundaryBoxes.ebbFLEFT_FOOT; i++)
		{
			local pbb = paniDino.pbbBoundaryBox(i);
			if (pbb)
				pbb.AISetRefType(eaiRefType);
		}
	}

	function GetAIType(paniDino)
	{
		// get the type from the first bbox that has one
		local i;
		for (i=EBoundaryBoxes.ebbBODY; i<=EBoundaryBoxes.ebbFLEFT_FOOT; i++)
		{
			local pbb = paniDino.pbbBoundaryBox(i);
			if (pbb)
			{
				local type = pbb.AIGetRefType();
				if (type != null)
					return type;
			}
		}

		// for unknown reason it failed to get the type, default to animal
		return EAIRefType.eaiANIMAL;
	}
}
